package com.hncy.sjdz.base.service.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.util.StringUtil;
import com.hncy.sjdz.base.models.entity.SjdzInterfaceInfo;
import com.hncy.sjdz.base.mapper.SjdzInterfaceInfoMapper;
import com.hncy.sjdz.base.models.vo.InterfaceVO;
import com.hncy.sjdz.base.service.SjdzInterfaceInfoService;
import com.hncy.sjdz.util.JsonUtils;
import com.hncy4cloud.hncy.common.core.util.R;
import com.troy.keeper.utils.SM4Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 智慧城管数据底座接口
 *
 * @author zhaoting
 * @date 2024-12-03 10:53:33
 */
@Service
@Slf4j
public class SjdzInterfaceInfoServiceImpl extends ServiceImpl<SjdzInterfaceInfoMapper, SjdzInterfaceInfo> implements SjdzInterfaceInfoService {

	@Override
	public R getSjdzInfoData(InterfaceVO interfaceVO) {
		String typeCode = interfaceVO.getTypeCode();
		if(StringUtil.isEmpty(typeCode)){
			return R.failed("接口类型不能为空");
		}else {
			List<SjdzInterfaceInfo> sjdzInterfaceInfos = baseMapper.selectList(Wrappers.<SjdzInterfaceInfo>lambdaQuery().eq(SjdzInterfaceInfo::getTypeCode, typeCode));
			if(sjdzInterfaceInfos.size()==0){
				return R.failed("未查询到该类型接口信息");
			}
			SjdzInterfaceInfo sjdzInterfaceInfo = sjdzInterfaceInfos.get(0);
			String interfaceAddress = sjdzInterfaceInfo.getInterfaceAddress();
			String method = sjdzInterfaceInfo.getMethod();
			String authUserName = sjdzInterfaceInfo.getAuthUserName();
			String authUserKey = sjdzInterfaceInfo.getAuthUserKey();
			String id = interfaceVO.getId();
			String url= interfaceAddress;
			if(StringUtil.isNotEmpty(id)){
				url=url+"/"+id;
			}

			HttpRequest request = HttpUtil.createRequest(cn.hutool.http.Method.valueOf(method), url);
			Map<String, Object> params = interfaceVO.getParams();
			if(params!=null){
				request.form(params);
			}
			//todo //测试
			//request.header("Authorization","Bearer e5353e80-a94a-43ae-8726-9c2fd4888119");
			Map<String, Object> data = interfaceVO.getData();
			if(data!=null){
				String s = JsonUtils.toJson(data);
				request.body(s);
			}
			if(StringUtil.isNotEmpty(authUserName)){
				request.header("authUserName",authUserName);
			}
			if(StringUtil.isNotEmpty(authUserKey)){
				request.header("authUserKey",authUserKey);
			}
			String result  = request.execute().body();
			log.info("result:{}",result);
			String isDecryption = sjdzInterfaceInfo.getIsDecryption();
			if("1".equals(isDecryption)){
				String privateKey = sjdzInterfaceInfo.getPrivateKey();
				try {
					result = SM4Util.decryptEcb(privateKey,result);
					JSONObject entries = JSONUtil.parseObj(result,false);
					return R.ok(entries);
				} catch (Exception e) {
					e.printStackTrace();
					return R.failed("解密失败");
				}
			}else {
				//不需要解密
				JSONObject entries = JSONUtil.parseObj(result,false);
				return R.ok(entries);
			}
		}
	}
}